from datetime import timedelta
from unittest import mock

from django.contrib.auth.models import User
from django.core import mail
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.utils import timezone
from freezegun import freeze_time

from .. import models

try:
    try:
        from psycopg.types.range import Range as DateTimeTZRange
    except ImportError:
        from psycopg2.extras import DateTimeTZRange
except ImportError:
    raise ImproperlyConfigured("Error loading psycopg2 or psycopg module")


@freeze_time("2019-10-03T12:00:00-04:00")
class ReportTest(TestCase):
    def setUp(self) -> None:
        self.staff_user = User.objects.create_user(
            username="root", email="root@example.com", is_staff=True
        )

        self.report = models.Report()
        self.report.uuid = "d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c"
        self.report.content_type = None
        self.report.owner = self.staff_user
        self.report.type_code = "order_report"
        self.report.date_range = DateTimeTZRange(
            lower=(timezone.now() - timedelta(days=1)), upper=(timezone.now())
        )
        self.report.save()

    def test_str(self) -> None:
        self.assertEqual(str(self.report), "d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c")

    def test_status(self) -> None:
        self.report.queued_on = None
        self.report.started_on = None
        self.report.completed_on = None
        self.report.save()
        self.assertEqual(self.report.status, "created")
        self.assertEqual(self.report.status_name, "Created")
        self.assertFalse(self.report.is_complete)

        self.report.queued_on = timezone.now()
        self.report.save()
        self.assertEqual(self.report.status, "queued")
        self.assertEqual(self.report.status_name, "Queued")
        self.assertFalse(self.report.is_complete)

        self.report.started_on = timezone.now()
        self.report.save()
        self.assertEqual(self.report.status, "in-progress")
        self.assertEqual(self.report.status_name, "In-Progress")
        self.assertFalse(self.report.is_complete)

        self.report.completed_on = timezone.now()
        self.report.save()
        self.assertEqual(self.report.status, "completed")
        self.assertEqual(self.report.status_name, "Completed")
        self.assertTrue(self.report.is_complete)

    def test_generator_class(self) -> None:
        self.assertEqual(self.report.generator_class.description, "Orders placed")

    def test_generator_class_missing(self) -> None:
        self.report.type_code = "foo_bar_baz"
        with self.assertRaises(ValueError):
            print(self.report.generator_class)

    @mock.patch("oscarreports.tasks.generate_report")
    def test_queue(self, mock_generate_report: mock.MagicMock) -> None:
        task = mock.MagicMock()
        task.id = "f3a0b0a0-148a-4ba1-9ed4-dd5543e77d73"
        mock_generate_report.enqueue.return_value = task

        self.assertEqual(mock_generate_report.enqueue.call_count, 0)

        self.assertEqual(self.report.description, "")
        self.assertIsNone(self.report.queued_on)
        self.assertIsNone(self.report.started_on)
        self.assertIsNone(self.report.completed_on)
        self.assertIsNone(self.report.task_id)

        with self.captureOnCommitCallbacks(execute=True):
            self.report.queue()

        self.assertEqual(mock_generate_report.enqueue.call_count, 1)
        mock_generate_report.enqueue.assert_called_once_with(
            "d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c",
            "CSV",
        )

        self.assertTrue(self.report.description.startswith("Orders placed between"))
        self.assertIsNotNone(self.report.queued_on)
        self.assertIsNone(self.report.started_on)
        self.assertIsNone(self.report.completed_on)
        self.assertEqual(self.report.task_id, "f3a0b0a0-148a-4ba1-9ed4-dd5543e77d73")

    def test_generate(self) -> None:
        self.assertIsNone(self.report.started_on)
        self.assertIsNone(self.report.mime_type)
        self.assertIsNone(self.report.report_file.name)
        self.assertIsNone(self.report.completed_on)

        self.report.generate()

        self.assertIsNotNone(self.report.started_on)
        self.assertEqual(self.report.mime_type, "text/csv")
        self.assertTrue(
            self.report.report_file.name.startswith(
                "oscar-reports/2019/10/03/d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c"
            )
        )
        self.assertTrue(self.report.report_file.name.endswith(".csv"))
        self.assertIsNotNone(self.report.completed_on)

    def test_queue_integration(self) -> None:
        self.assertEqual(self.report.description, "")
        self.assertIsNone(self.report.queued_on)
        self.assertIsNone(self.report.started_on)
        self.assertIsNone(self.report.completed_on)
        self.assertIsNone(self.report.task_id)
        self.assertIsNone(self.report.mime_type)
        self.assertIsNone(self.report.report_file.name)
        self.assertEqual(len(mail.outbox), 0)

        with self.captureOnCommitCallbacks(execute=True):
            self.report.queue()

        self.report.refresh_from_db()

        self.assertTrue(self.report.description.startswith("Orders placed between"))
        self.assertIsNotNone(self.report.queued_on)
        self.assertIsNotNone(self.report.started_on)
        self.assertIsNotNone(self.report.completed_on)
        self.assertIsNotNone(self.report.task_id)
        self.assertEqual(self.report.mime_type, "text/csv")
        self.assertTrue(
            self.report.report_file.name.startswith(
                "oscar-reports/2019/10/03/d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c"
            )
        )
        self.assertTrue(self.report.report_file.name.endswith(".csv"))

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(
            mail.outbox[0].subject,
            "Your report is ready to download | Orders placed between Oct. 2, 2019 and Oct. 3, 2019",
        )

    def test_delete(self) -> None:
        self.report.generate()
        self.report.report_file.delete = mock.MagicMock()

        self.assertEqual(self.report.report_file.delete.call_count, 0)

        self.report.delete()

        self.assertEqual(self.report.report_file.delete.call_count, 1)
        self.report.report_file.delete.assert_called_once_with(save=False)



from __future__ import annotations

from typing import Any
import os.path

from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.db.models.query import QuerySet
from django.forms.models import BaseModelForm
from django.http import (
    FileResponse,
    Http404,
    HttpRequest,
    HttpResponse,
    HttpResponseBase,
)
from django.shortcuts import redirect
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic.detail import BaseDetailView
from django.views.generic.edit import DeleteView, FormMixin
from django_tables2 import SingleTableView

from .forms import ReportForm
from .models import Report
from .tables import ReportTable

try:
    try:
        from psycopg.types.range import Range as DateTimeTZRange
    except ImportError:
        from psycopg2.extras import DateTimeTZRange
except ImportError:
    raise ImproperlyConfigured("Error loading psycopg2 or psycopg module")


class IndexView(FormMixin[ReportForm], SingleTableView):
    template_name = "oscar/dashboard/reports/index.html"
    table_pagination = True  # type:ignore[assignment]
    model = Report
    table_class = ReportTable  # type:ignore[assignment]
    context_table_name = "reports"
    desc_template = ""
    description = ""

    form_class = ReportForm

    def dispatch(
        self,
        request: HttpRequest,
        *args: Any,
        **kwargs: Any,
    ) -> HttpResponseBase:
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self) -> QuerySet[Report]:
        queryset = self.model.objects.all().order_by("-queued_on")
        return queryset

    def get_table_pagination(self, table: ReportTable) -> dict[str, int]:
        return dict(per_page=20)

    def post(
        self,
        request: HttpRequest,
        *args: Any,
        **kwargs: Any,
    ) -> HttpResponse:
        if self.form.is_valid():
            # Create report
            report = Report()
            report.content_type = None
            report.owner = request.user if request.user.is_authenticated else None
            report.type_code = self.form.cleaned_data["report_type"]
            report.date_range = DateTimeTZRange(
                lower=self.form.cleaned_data["date_from"],
                upper=self.form.cleaned_data["date_to"],
            )
            report.save()
            # Queue report
            report.queue()
            # Message user
            messages.info(request, _("Successfully queued report for generation"))
            return redirect("dashboard:reports-index")
        return self.get(request, *args, **kwargs)

    def get_table(self, **kwargs: Any) -> ReportTable:
        table = super().get_table(**kwargs)
        table.caption = _("Reports")
        return table

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context["form"] = self.form
        return context


class ReportDownloadView(BaseDetailView[Report]):
    model = Report
    slug_field = "uuid"
    slug_url_kwarg = "uuid"

    def render_to_response(
        self,
        context: dict[str, Any],
        **response_kwargs: Any,
    ) -> FileResponse:
        report = context["object"]
        if not report.report_file.name:
            raise Http404()
        # Build filename
        extension = os.path.splitext(report.report_file.name)[1].replace(".", "")
        filename = "{date}_{type_code}_{uuid}.{ext}".format(
            date=report.created_on.strftime("%Y-%m-%d"),
            type_code=report.type_code,
            uuid=report.uuid,
            ext=extension,
        )
        # Send file to client
        resp = FileResponse(report.report_file, as_attachment=True, filename=filename)
        resp["Content-Type"] = report.mime_type
        return resp


class ReportDeleteView(DeleteView[Report, BaseModelForm[Report]]):
    model = Report
    slug_field = "uuid"
    slug_url_kwarg = "uuid"
    success_url = reverse_lazy("dashboard:reports-index")
    template_name = "oscar/dashboard/reports/report_confirm_delete.html"



from django.apps import apps
from django.conf import settings
from django.conf.urls import i18n as i18n_urls
from django.contrib import admin
from django.urls import include, path
from django.views.static import serve

oscar = apps.get_app_config("oscar")

urlpatterns = [
    path("i18n/", include(i18n_urls)),
    path("admin/", admin.site.urls),
    path(
        "media/<path:path>",
        serve,
        {"document_root": settings.MEDIA_ROOT, "show_indexes": True},
    ),
    # Include Oscar
    path("", include(oscar.urls[0])),  # type:ignore[attr-defined]
]



from __future__ import annotations

from collections.abc import Callable
from enum import StrEnum
from typing import TYPE_CHECKING, Generic, assert_never
from uuid import UUID

from django.conf import settings

if TYPE_CHECKING:
    from celery.app.task import Task as _CeleryTask
    from celery.result import AsyncResult as _CeleryAsyncResult

    try:
        from django_tasks.base import Task as _DjTask
        from django_tasks.base import TaskResult as _DjTaskResult
    except ImportError:  # django-tasks prior to 0.9.0
        from django_tasks.task import Task as _DjTask  # type:ignore
        from django_tasks.task import TaskResult as _DjTaskResult  # type:ignore
else:
    _CeleryTask = Callable
    _CeleryAsyncResult = Generic
    _DjTask = Callable
    _DjTaskResult = Generic


class BackendType(StrEnum):
    CELERY = "celery"
    DJANGO_TASKS = "django-tasks"


class TaskFuture[T]:
    @property
    def id(self) -> str:
        raise NotImplementedError()

    @property
    def status_label(self) -> str | None:
        raise NotImplementedError()


class CeleryTaskFuture[T](TaskFuture[T]):
    _res: _CeleryAsyncResult[T]

    def __init__(self, res: _CeleryAsyncResult[T]) -> None:
        self._res = res

    @property
    def id(self) -> str:
        return str(self._res.id)

    @property
    def status_label(self) -> str | None:
        status = self._res.status
        return str(status).title()


class DjTasksTaskFuture[T](TaskFuture[T]):
    _res: _DjTaskResult[T]

    def __init__(self, res: _DjTaskResult[T]) -> None:
        self._res = res

    @property
    def id(self) -> str:
        return self._res.id

    @property
    def status_label(self) -> str | None:
        status = self._res.status
        # If enum, use name; else cast to string
        return getattr(status, "name", str(status)).title()


class Task[**P, T]:
    backend_type: BackendType

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
        raise NotImplementedError()

    def enqueue(self, *args: P.args, **kwargs: P.kwargs) -> TaskFuture[T]:
        raise NotImplementedError()

    def get_result(self, task_id: str) -> TaskFuture[T]:
        raise NotImplementedError()


class CeleryTask[**P, T](Task[P, T]):
    fn: _CeleryTask[P, T]

    def __init__(self, fn: Callable[P, T]) -> None:
        from celery import shared_task

        self.fn = shared_task(ignore_result=True)(fn)

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
        return self.fn(*args, **kwargs)

    def enqueue(self, *args: P.args, **kwargs: P.kwargs) -> CeleryTaskFuture[T]:
        res = self.fn.apply_async(
            args=args,
            kwargs=kwargs,
            countdown=10,
        )
        return CeleryTaskFuture(res)

    def get_result(self, task_id: str) -> CeleryTaskFuture[T]:
        from celery.result import AsyncResult

        return CeleryTaskFuture[T](AsyncResult(task_id))


class DjTasksTask[**P, T](Task[P, T]):
    fn: _DjTask[P, T]

    def __init__(self, fn: Callable[P, T]) -> None:
        from django_tasks import task

        self.fn = task()(fn)

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
        return self.fn.call(*args, **kwargs)

    def enqueue(self, *args: P.args, **kwargs: P.kwargs) -> DjTasksTaskFuture[T]:
        res = self.fn.enqueue(*args, **kwargs)
        return DjTasksTaskFuture(res)

    def get_result(self, task_id: str) -> DjTasksTaskFuture[T]:
        res = self.fn.get_result(task_id)
        return DjTasksTaskFuture(res)


def task[**P, T](fn: Callable[P, T]) -> Task[P, T]:
    btype_str = getattr(settings, "OSCAR_REPORTS_TASKS_BACKEND", None)
    if btype_str is None:
        try:
            import celery  # NOQA

            btype_str = "celery"
        except ImportError:
            try:
                import django_tasks  # NOQA

                btype_str = "django-tasks"
            except ImportError:
                btype_str = "unknown"

    btype = BackendType(btype_str)
    if btype == BackendType.CELERY:
        return CeleryTask(fn)
    if btype == BackendType.DJANGO_TASKS:
        return DjTasksTask(fn)
    assert_never(btype)


@task
def generate_report(report_uuid: str, report_format: str) -> None:
    from . import models

    report = models.Report.objects.get(uuid=UUID(report_uuid))
    report.generate(report_format)



# Generated by Django 5.2.1 on 2025-06-11 22:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("reports_dashboard", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="report",
            name="task_id",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=64,
                null=True,
                unique=True,
                verbose_name="Background Task ID",
            ),
        ),
    ]



from django.contrib import admin

from . import models


@admin.register(models.Report)
class ReportAdmin(admin.ModelAdmin[models.Report]):
    search_fields = ["uuid"]
    raw_id_fields = ["owner"]
    list_display = [
        "uuid",
        "owner",
        "content_type",
        "type_code",
        "created_on",
        "queued_on",
        "started_on",
        "completed_on",
        "status_name",
    ]
    list_filter = [
        "type_code",
        "created_on",
        "queued_on",
        "started_on",
        "completed_on",
    ]
    readonly_fields = [
        "uuid",
        "task_id",
        "created_on",
    ]
    fields = [
        "uuid",
        "content_type",
        "type_code",
        "owner",
        "description",
        "date_range",
        "task_id",
        "created_on",
        "queued_on",
        "started_on",
        "completed_on",
        "mime_type",
        "report_file",
    ]



from django.core.exceptions import ImproperlyConfigured
from oscar.apps.dashboard.reports import utils
from oscar.apps.dashboard.reports.reports import ReportGenerator


class GeneratorRepository(utils.GeneratorRepository):
    generators: list[type[ReportGenerator]]

    @classmethod
    def register(cls, ReportGeneratorSubclass: type[ReportGenerator]) -> None:
        if ReportGeneratorSubclass in cls.generators:
            raise ImproperlyConfigured(
                "The class %s is already registered" % ReportGeneratorSubclass
            )
        cls.generators.append(ReportGeneratorSubclass)



from datetime import timedelta

from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from django.utils import timezone
from oscar.test.testcases import WebTestCase
from webtest.app import AppError

from .. import models

try:
    # For Oscar >=4.0
    from oscar.apps.dashboard.permissions import DashboardPermission

    _permissions = DashboardPermission.get("user_record")
except ImportError:
    _permissions = WebTestCase.permissions

try:
    try:
        from psycopg.types.range import Range as DateTimeTZRange
    except ImportError:
        from psycopg2.extras import DateTimeTZRange
except ImportError:
    raise ImproperlyConfigured("Error loading psycopg2 or psycopg module")


class ReportsDashboardTests(WebTestCase):
    is_staff = True
    permissions = _permissions

    def test_dashboard_is_accessible_to_staff(self) -> None:
        url = reverse("dashboard:reports-index")
        response = self.get(url)
        self.assertIsOk(response)

    def test_conditional_offers_no_date_range(self) -> None:
        url = reverse("dashboard:reports-index")
        response = self.get(url)

        form = response.forms["generate_report_form"]
        form["report_type"] = "conditional-offers"
        form.submit()
        self.assertIsOk(response)

    def test_conditional_offers_with_date_range(self) -> None:
        url = reverse("dashboard:reports-index")
        response = self.get(url)

        form = response.forms["generate_report_form"]
        form["report_type"] = "conditional-offers"
        form["date_from"] = "2017-01-01"
        form["date_to"] = "2017-12-31"
        form.submit()
        self.assertIsOk(response)

    def test_conditional_offers_with_invalid_date_range(self) -> None:
        url = reverse("dashboard:reports-index")
        response = self.get(url)

        form = response.forms["generate_report_form"]
        form["report_type"] = "conditional-offers"
        form["date_from"] = "2017-12-31"
        form["date_to"] = "2017-01-01"
        form.submit()
        self.assertIsOk(response)


class DownloadReportDashboardTests(WebTestCase):
    is_staff = True
    permissions = _permissions

    def setUp(self) -> None:
        super().setUp()
        self.staff_user = User.objects.create_user(
            username="root", email="root@example.com", is_staff=True
        )
        self.report = models.Report()
        self.report.uuid = "d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c"
        self.report.content_type = None
        self.report.owner = self.staff_user
        self.report.type_code = "order_report"
        self.report.date_range = DateTimeTZRange(
            lower=(timezone.now() - timedelta(days=1)), upper=(timezone.now())
        )
        self.report.save()

    def test_download_not_found(self) -> None:
        url = reverse("dashboard:reports-download", args=[self.report.uuid])
        with self.assertRaises(AppError):
            self.get(url)

    def test_download_report(self) -> None:
        self.report.generate()
        url = reverse("dashboard:reports-download", args=[self.report.uuid])
        response = self.get(url)
        self.assertIsOk(response)


class DeleteReportDashboardTests(WebTestCase):
    is_staff = True
    permissions = _permissions

    def setUp(self) -> None:
        super().setUp()
        self.staff_user = User.objects.create_user(
            username="root", email="root@example.com", is_staff=True
        )
        self.report = models.Report()
        self.report.uuid = "d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c"
        self.report.content_type = None
        self.report.owner = self.staff_user
        self.report.type_code = "order_report"
        self.report.date_range = DateTimeTZRange(
            lower=(timezone.now() - timedelta(days=1)), upper=(timezone.now())
        )
        self.report.save()

    def test_dashboard_is_accessible_to_staff(self) -> None:
        url = reverse("dashboard:reports-delete", args=[self.report.uuid])
        response = self.get(url)
        self.assertIsOk(response)

    def test_can_delete_report(self) -> None:
        self.assertEqual(models.Report.objects.count(), 1)
        url = reverse("dashboard:reports-delete", args=[self.report.uuid])
        response = self.get(url)
        form = response.forms["delete_report_form"]
        form.submit()
        self.assertIsOk(response)
        self.assertEqual(models.Report.objects.count(), 0)
